# Daikin KKRP01A Controller
#
# Author: foxfire_pt, 2017
#
"""
<plugin key="KKRP01A" name="Daikin Airconditioning (KKRP01A)" author="foxfire_pt" version="2.0.0" externallink="http://www.onlinecontroller.eu/">
    <params>
        <param field="Address" label="IP Address" width="200px" required="true" default=""/>
        <param field="Port" label="Port" width="30px" required="true" default="80"/>
        <param field="Mode1" label="Debug" width="75px">
            <options>
                <option label="True" value="Debug"/>
                <option label="False" value="Normal"  default="true" />
            </options>
        </param>
    </params>
</plugin>
"""
import Domoticz
import hmac
import hashlib
import time
import re
import base64
import http.client

class BasePlugin:
    enabled = False
    pluginState = "Not Ready"
    PowerOn = "OFF"
    TempIn  = 0
    TempOut = 0
    Humidity = 0
    Mode = "AUTO"
    Fan = "FA"
    TempAC = 0
    Swing= "OFF"
    Address="127.0.0.1"
    Port="80"
    def __init__(self):
        return

    #def stringToBase64(self, s):
    #    return base64.b64encode(s.encode('utf-8')).decode("utf-8")

    def onStart(self):
        if Parameters["Mode1"] == "Debug":
            Domoticz.Debugging(1)
        if (len(Devices) == 0):
            Domoticz.Device(Name="Power", Unit=1, Image=16, TypeName="Switch").Create()
            Domoticz.Device(Name="Temp IN", Unit=2, TypeName="Temp+Hum").Create()
            Domoticz.Device(Name="Temp OUT", Unit=3, TypeName="Temperature").Create()

            Options = {"LevelActions": "|||||", "LevelNames": "AUTO|DRY|COOL|HEAT|ONLYFUN|NIGHT", "LevelOffHidden": "false", "SelectorStyle": "1"}
            Domoticz.Device(Name="Mode", Unit=4, TypeName="Selector Switch", Image=16, Options=Options).Create()

            Options = {"LevelActions": "|||||", "LevelNames": "AUTO|F1|F2|F3|F4|F5", "LevelOffHidden": "false", "SelectorStyle": "1"}
            Domoticz.Device(Name="Fan", Unit=5, TypeName="Selector Switch", Image=16, Options=Options).Create()

            Options = {"LevelActions": "||||", "LevelNames": "10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31|32", "LevelOffHidden": "false", "SelectorStyle": "1"}
            Domoticz.Device(Name="Temp AC", Unit=6, TypeName="Selector Switch", Image=16, Options=Options).Create()

            Options = {"LevelActions": "||||", "LevelNames": "OFF|UP/DOWN", "LevelOffHidden": "false", "SelectorStyle": "1"}
            Domoticz.Device(Name="Swing", Unit=7, TypeName="Selector Switch", Image=16, Options=Options).Create()

            Domoticz.Log("Device created.")
        DumpConfigToLog()
        self.Address=Parameters["Address"]
        self.Port=Parameters["Port"]
        #Domoticz.Transport(Transport="TCP/IP", Address=Parameters["Address"], Port=Parameters["Port"])
        #Domoticz.Protocol("HTTP")
        Domoticz.Heartbeat(30)
        #Domoticz.Connect()
        if (1 in Devices) and (Devices[1].nValue == 1):
             self.PowerOn = "ON"
        #if (2 in Devices):
        #     self.TempIn = Devices[2].nValue
        if (3 in Devices):
             self.TempOut = Devices[3].nValue
        if (4 in Devices):
             self.Mode = Devices[4].nValue

    def onStop(self):
        Domoticz.Log("Plugin is stopping.")

    def onHeartbeat(self):
        Domoticz.Debug("onHeartbeat called")
        conn = http.client.HTTPConnection( self.Address, port=self.Port, timeout=3)
        conn.request("GET", "/param.csv")
        httpResponse = conn.getresponse()
        if httpResponse.status == 200:
          Domoticz.Log("GET responce was "+str(httpResponse.status)+" for "+self.Address+":"+self.Port)
          strData =  httpResponse.read().decode('utf-8')
          if re.match("^HTTP",strData) is not None:
             (rlist)=strData.split("\r\n\r\n")
             strData=rlist[1]
          (rlist)=strData.split(".\r\n");
          if ( rlist[1] == "OFF" ):
             (tlist)=rlist[12].split(".")
             rlist[2]=tlist[0]
             rlist[3]=tlist[2]
             rlist[4]=tlist[4]
          rlist[6]=rlist[6].replace(",", ".")
          rlist[14]=rlist[14].replace(",", ".")
          if  ( rlist[0] == "OK" ):
             if ( len(rlist[1] ) and (self.PowerOn != rlist[1]) ):
                   self.PowerOn=rlist[1]
                   Domoticz.Log("Power State has changed, Now: "+self.PowerOn)
                   if (self.PowerOn == "ON"):
                        if (1 in Devices): Devices[1].Update(1,"100")
                   else:
                        if (1 in Devices): Devices[1].Update(0,"0")
             if  ( len(rlist[6]) and (self.TempIn != rlist[6]) ):
                   self.TempIn=rlist[6]
                   tmp=str(self.TempIn)+";"+str(self.Humidity)+";1"
                   if (2 in Devices): Devices[2].Update(1,tmp)
                   Domoticz.Log("Temperature In State has changed, Now: "+self.TempIn)
             if  ( len(rlist[14]) and (self.TempOut != rlist[14]) ):
                   self.TempOut=rlist[14]
                   if (3 in Devices): Devices[3].Update(0,self.TempOut)
                   Domoticz.Log("Temperature Out State has changed, Now: "+self.TempOut)
             if  ( len(rlist[15]) and (self.Humidity != rlist[15]) ):
                   self.Humidity=rlist[15]
                   tmp=str(self.TempIn)+";"+str(self.Humidity)+";1"
                   if (2 in Devices): Devices[2].Update(1,tmp)
                   Domoticz.Log("Humidity State has changed, Now: "+self.Humidity)
             if  ( len(rlist[2]) and (self.Mode != rlist[2]) ):
                   self.Mode=rlist[2]
                   if (4 in Devices):
                        if   ( self.Mode == "AUTO" ):
                            if (4 in Devices): Devices[4].Update(1,"0")
                        elif ( self.Mode == "DRY" ):
                            if (4 in Devices): Devices[4].Update(1,"10")
                        elif ( self.Mode == "COOL" ):
                            if (4 in Devices): Devices[4].Update(1,"20")
                        elif ( self.Mode == "HEAT" ):
                            if (4 in Devices): Devices[4].Update(1,"30")
                        elif ( self.Mode == "ONLYFUN" ):
                            if (4 in Devices): Devices[4].Update(1,"40")
                        elif( self.Mode == "NIGHT" ):
                            if (4 in Devices): Devices[4].Update(1,"50")
                   Domoticz.Log("Mode State has changed, Now: "+self.Mode)
             if  ( len(rlist[4]) and (self.Fan != rlist[4]) ):
                   self.Fan=rlist[4]
                   if (5 in Devices):
                        if   ( self.Fan == "FA" ):
                            if (5 in Devices): Devices[5].Update(1,"0")
                        elif ( self.Fan == "F1" ):
                            if (5 in Devices): Devices[5].Update(1,"10")
                        elif ( self.Fan == "F2" ):
                            if (5 in Devices): Devices[5].Update(1,"20")
                        elif ( self.Fan == "F3" ):
                            if (5 in Devices): Devices[5].Update(1,"30")
                        elif ( self.Fan == "F4" ):
                            if (5 in Devices): Devices[5].Update(1,"40")
                        elif( self.Fan == "F5" ):
                            if (5 in Devices): Devices[5].Update(1,"50")
                   Domoticz.Log("Fan State has changed, Now: "+self.Fan)
             if  ( len(rlist[3]) and (self.TempAC != rlist[3]) ):
                   self.TempAC=rlist[3]
                   if (6 in Devices):
                        tmp=(int(self.TempAC)-10)*10
                        tmp=str(tmp)
                        if (6 in Devices): Devices[6].Update(1,tmp)
                   Domoticz.Log("TempAC State has changed, Now: "+self.TempAC)
             if  ( len(rlist[5]) and (self.Swing != rlist[5]) ):
                   self.Swing=rlist[5]
                   if (7 in Devices):
                        if   ( self.Swing == "OFF" ):
                            if (7 in Devices): Devices[7].Update(1,"0")
                        elif ( self.Swing == "UD" ):
                            if (7 in Devices): Devices[7].Update(1,"10")
                   Domoticz.Log("Swing State has changed, Now: "+self.Swing)
          else:
             Domoticz.Log("KKRP01A Bad Responce from "+Parameters["Address"]+":"+Parameters["Port"])
        else:
          Domoticz.Log("KKRP01A HTTP ERROR "+str(httpResponse.status)+" from "+Parameters["Address"]+":"+Parameters["Port"])

    def onCommand(self, Unit, Command, Level, Hue):
        Domoticz.Debug("Command received U="+str(Unit)+" C="+str(Command)+" L= "+str(Level)+" H= "+str(Hue) )

        if (Unit  == 1 ):
             if ( Command == "On" ):
                  self.PowerOn="ON";
                  if (1 in Devices): Devices[1].Update(1,"100")
             elif ( Command == "Off" ):
                  self.PowerOn="OFF";
                  if (1 in Devices): Devices[1].Update(0,"0")
        if (Unit  == 4 ):
             if   ( Level == 0 ):
                  self.Mode="AUTO"
                  if (4 in Devices): Devices[4].Update(1,"0")
             elif ( Level == 10 ):
                  self.Mode="DRY"
                  if (4 in Devices): Devices[4].Update(1,"10")
             elif ( Level == 20 ):
                  self.Mode="COOL"
                  if (4 in Devices): Devices[4].Update(1,"20")
             elif ( Level == 30 ):
                  self.Mode="HEAT"
                  if (4 in Devices): Devices[4].Update(1,"30")
             elif ( Level == 40 ):
                  self.Mode="ONLYFUN"
                  if (4 in Devices): Devices[4].Update(1,"40")
             elif ( Level == 50 ):
                  self.Mode="NIGHT"
                  if (4 in Devices): Devices[4].Update(1,"50")

        if (Unit  == 5 ):
             if   ( Level == 0 ):
                  self.Fan="FA"
                  if (5 in Devices): Devices[5].Update(1,"0")
             elif ( Level == 10 ):
                  self.Fan="F1"
                  if (5 in Devices): Devices[5].Update(1,"10")
             elif ( Level == 20 ):
                  self.Fan="F2"
                  if (5 in Devices): Devices[5].Update(1,"20")
             elif ( Level == 30 ):
                  self.Fan="F3"
                  if (5 in Devices): Devices[5].Update(1,"30")
             elif ( Level == 40 ):
                  self.Fan="F4"
                  if (5 in Devices): Devices[5].Update(1,"40")
             elif ( Level == 50 ):
                  self.Fan="F5"
                  if (5 in Devices): Devices[5].Update(1,"50")

        if (Unit  == 6 ):
              tmp=int(int(Level)/10+10)
              self.TempAC=str(tmp)
              if (6 in Devices): Devices[6].Update(1,str(Level))
        if (Unit  == 7 ):
             if   ( Level == 0 ):
                  self.Swing="OFF"
                  if (7 in Devices): Devices[7].Update(1,"0")
             elif ( Level == 10 ):
                  self.Swing="UD"
                  if (7 in Devices): Devices[7].Update(1,"10")

        poststr="wON="
        if ( self.PowerOn == "OFF" ):
             poststr=poststr+"Off"
        elif ( self.PowerOn == "ON" ):
             poststr=poststr+"On"

        poststr=poststr+"&wMODE="
        if   ( self.Mode == "AUTO" ):
             poststr=poststr+"Auto"
        elif ( self.Mode == "DRY" ):
             poststr=poststr+"Dry"
        elif ( self.Mode == "COOL" ):
             poststr=poststr+"Cool"
        elif ( self.Mode == "HEAT" ):
             poststr=poststr+"Heat"
        elif ( self.Mode == "ONLYFUN" ):
             poststr=poststr+"OnlyFun"
        elif ( self.Mode == "NIGHT" ):
             poststr=poststr+"Night"

        poststr=poststr+"&wTEMP="+str(self.TempAC)+"C"

        poststr=poststr+"&wFUN="
        if   ( self.Fan == "FA" ):
             poststr=poststr+"FAuto"
        elif ( self.Fan == "F1" ):
             poststr=poststr+"Fun1"
        elif ( self.Fan == "F2" ):
             poststr=poststr+"Fun2"
        elif ( self.Fan == "F3" ):
             poststr=poststr+"Fun2"
        elif ( self.Fan == "F4" ):
             poststr=poststr+"Fun2"
        elif ( self.Fan == "F5" ):
             poststr=poststr+"Fun2"

        poststr=poststr+"&wSWNG="
        if   ( self.Swing == "OFF" ):
             poststr=poststr+"Off"
        elif ( self.Swing == "UD" ):
             poststr=poststr+"UD"

        poststr=poststr+"&wSETd1=Set"
        Domoticz.Debug("Command  "+poststr)

        headers = { 'Content-Type': 'text/xml; charset=utf-8' }
        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
        conn = http.client.HTTPConnection( self.Address, port=self.Port)
        conn.request("POST", "/index.htm", poststr, headers)
        httpResponse = conn.getresponse()
        Domoticz.Log("Post responce was "+str(httpResponse.status)+" for "+poststr)


global _plugin
_plugin = BasePlugin()

def onStart():
    global _plugin
    _plugin.onStart()

def onStop():
    global _plugin
    _plugin.onStop()

def stringToBase64(s):
    global _plugin
    _plugin.stringToBase64(s)

def onHeartbeat():
    global _plugin
    _plugin.onHeartbeat()

def onCommand(Unit, Command, Level, Hue):
    global _plugin
    _plugin.onCommand(Unit, Command, Level, Hue)

# Generic helper functions

def stringToBase64(s):
    return base64.b64encode(s.encode('utf-8')).decode("utf-8")

def DumpConfigToLog():
    for x in Parameters:
        if Parameters[x] != "":
            Domoticz.Debug( "'" + x + "':'" + str(Parameters[x]) + "'")
    Domoticz.Debug("Device count: " + str(len(Devices)))
    for x in Devices:
        Domoticz.Debug("Device:           " + str(x) + " - " + str(Devices[x]))
        Domoticz.Debug("Device ID:       '" + str(Devices[x].ID) + "'")
        Domoticz.Debug("Device Name:     '" + Devices[x].Name + "'")
        Domoticz.Debug("Device nValue:    " + str(Devices[x].nValue))
        Domoticz.Debug("Device sValue:   '" + Devices[x].sValue + "'")
        Domoticz.Debug("Device LastLevel: " + str(Devices[x].LastLevel))
    return
